/*----------------------------------------------------------------------
  Prefs.c

  Copyright (C). 2003 Smoky Monkey Factory All Rights Reserved
  http://andmore.jp/mobile/

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  $Id: Prefs.c,v 1.2 2003/07/29 15:09:11 zxn Exp $
  ----------------------------------------------------------------------*/

#include <PalmOS.h>
#include "Prefs.h"
#include "SnapSwitch.h"
#include "Etc.h"

#define KEY_LIST_MAX 5

typedef struct
{
  UInt16 card_no;
  LocalID db_id;
} appInfoType;

#define RECENT_APPS_MAX 8

typedef struct 
{
  Boolean appEnabled;
  prefsAutoActivateIdx autoActivate;
  Boolean ignore;
  UInt32 time;

  UInt16 key_idx;
  UInt16 key_list[KEY_LIST_MAX];

  appInfoType app_info_list[RECENT_APPS_MAX];
  UInt16 app_info_idx;

} prefsType;

static void prefs_Init(prefsType *prefs_p);

/*-----------------------------------------------------------------------
  new & delete
  -----------------------------------------------------------------------*/

prefsObj prefs_New(void)
{
  prefsObj prefs = (prefsObj)MemPtrNew(sizeof(prefsType));
  if (prefs != NULL)
    prefs_Init((prefsType *)prefs);
  return prefs;
}

void prefs_Delete(prefsObj prefs)
{
  MemPtrFree(prefs);
}


/*-----------------------------------------------------------------------
  load prefs, or init it 
  -----------------------------------------------------------------------*/
static void prefs_Init(prefsType *prefs_p)
{
  if (PrefGetAppPreferencesV10(CREATOR_ID, 
			       PREF_VERSION, 
			       (void *)(prefs_p),
			       sizeof(prefsType)) == false) 
    {
      UInt16 card_no;
      LocalID db_id;
      UInt32 i;

      prefs_p->appEnabled = false;
      prefs_p->autoActivate = PREFS_AUTO_ACTIVATE_NO;

      /* app list */
      for (i=0; i<RECENT_APPS_MAX; i++)
	{
	  prefs_p->app_info_list[i].card_no = 0;
	  prefs_p->app_info_list[i].db_id = 0;
	}

      /* at least one app should be set xxxx */
      SysCurAppDatabase(&card_no, &db_id);

      prefs_p->app_info_list[0].card_no = card_no;
      prefs_p->app_info_list[0].db_id = db_id;

      prefs_p->app_info_idx = 1;	
      prefs_p->ignore = false;	
      prefs_p->time = 0;	
      prefs_p->key_idx = 0;	
      
      prefs_p->key_list[0] = vchrHard1;
      prefs_p->key_list[1] = vchrHard2;
      prefs_p->key_list[2] = vchrHard3;
      prefs_p->key_list[3] = vchrHard4;
      prefs_p->key_list[4] = vchrLaunch;
    }
}




/*-----------------------------------------------------------------------
  prefsEnabled
  -----------------------------------------------------------------------*/

Boolean prefs_GetEnabled(prefsObj prefs)
{
  return ((prefsType *)prefs)->appEnabled;
}

void prefs_SetEnabled(prefsObj prefs, Boolean newVal)
{
  ((prefsType *)prefs)->appEnabled = newVal;
}

/*-----------------------------------------------------------------------
  auto activate after reset
  -----------------------------------------------------------------------*/

prefsAutoActivateIdx prefs_GetAutoActivate(prefsObj prefs)
{
  return ((prefsType *)prefs)->autoActivate;
}

void prefs_SetAutoActivate(prefsObj prefs, prefsAutoActivateIdx stat)
{
  ((prefsType *)prefs)->autoActivate = stat;
}


/*-----------------------------------------------------------------------
  my launch
  -----------------------------------------------------------------------*/

Boolean prefs_GetIgnore(prefsObj prefs)
{
  return ((prefsType *)prefs)->ignore;
}

void prefs_SetIgnore(prefsObj prefs, Boolean val)
{
  ((prefsType *)prefs)->ignore = val;
}


/*-----------------------------------------------------------------------
  app info
  -----------------------------------------------------------------------*/
void prefs_AppendAppToQueue(prefsObj prefs, UInt16 card_id, LocalID db_id)
{
  UInt16 app_info_idx = ((prefsType *)prefs)->app_info_idx;
  appInfoType *app_info_list = ((prefsType *)prefs)->app_info_list;	

  /* set app info */
  app_info_list[app_info_idx].card_no = card_id;
  app_info_list[app_info_idx].db_id = db_id;

  ((prefsType *)prefs)->app_info_idx++;
  if (((prefsType *)prefs)->app_info_idx == RECENT_APPS_MAX)
    ((prefsType *)prefs)->app_info_idx = 0;
}

Boolean prefs_GetAppFromQueue(prefsObj prefs, UInt16 *card_no_p, LocalID *db_id_p)
{
  UInt16 app_info_idx = ((prefsType *)prefs)->app_info_idx;
  appInfoType *app_info_list = ((prefsType *)prefs)->app_info_list;	

  if (app_info_idx == 0)
    app_info_idx = RECENT_APPS_MAX - 1;
  else
    app_info_idx--;

  /* set app info */
  *card_no_p = app_info_list[app_info_idx].card_no;
  *db_id_p = app_info_list[app_info_idx].db_id;

#if 0
  /* remove app from app info list  */
  app_info_list[app_info_idx].card_no = 0;
  app_info_list[app_info_idx].db_id = 0;
#endif

  /* update index */
  ((prefsType *)prefs)->app_info_idx = app_info_idx;

  /* xxxx */
  if (*card_no_p == 0 && *db_id_p == 0)
    return false;
  else
    return true;

}


/*-----------------------------------------------------------------------
  save prefs
  -----------------------------------------------------------------------*/
void prefs_Save(prefsObj prefs)
{
  PrefSetAppPreferencesV10(CREATOR_ID, 
			   PREF_VERSION, 
			   (void *)prefs, 
			   sizeof(prefsType));
}


/*-----------------------------------------------------------------------
  time
  -----------------------------------------------------------------------*/

UInt32 prefs_GetTime(prefsObj prefs)
{
  return ((prefsType *)prefs)->time;
}

void prefs_SetTime(prefsObj prefs, UInt32 val)
{
  ((prefsType *)prefs)->time = val;
}

/*-----------------------------------------------------------------------
  key list 
  -----------------------------------------------------------------------*/

void prefs_InitKeyList(prefsObj prefs)
{
  ((prefsType *)prefs)->key_idx = 0;
}

UInt16 prefs_GetNextKey(prefsObj prefs)
{
  
  UInt16 chr;
  UInt16 idx =  ((prefsType *)prefs)->key_idx;
  chr = ((prefsType *)prefs)->key_list[idx];

  idx++;
  if (idx == KEY_LIST_MAX)
    idx = 0;

  ((prefsType *)prefs)->key_idx = idx;

  return chr;
}
